
#  ============LICENSE_START===============================================
#  Copyright (C) 2021 Nordix Foundation. All rights reserved.
#  ========================================================================
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#  ============LICENSE_END=================================================
#

from flask import Flask
from flask import request

import requests
import json

import threading
import time
from datetime import datetime
import logging
import os
import sys
import traceback
import random
from requests.packages import urllib3

# Remove warning fot https calls
urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)

app = Flask(__name__)

#Set to True to log att exception traces
debug=False

# Server info
HOST_IP = "::"
HOST_PORT = 2222

# Url for exposed web interface and callback urls
CALLBACK_JOB_RESULT_BASE="/callbacks/job-result"
CALLBACK_JOB_RESULT=CALLBACK_JOB_RESULT_BASE+"/<string:job_id>"

CALLBACK_JOB_STATUS_BASE="/callbacks/job-status"
CALLBACK_JOB_STATUS=CALLBACK_JOB_STATUS_BASE+"/<string:job_id>"

CALLBACK_SERVICE_BASE="/callbacks/service-status"
CALLBACK_SERVICE=CALLBACK_SERVICE_BASE+"/<string:service_id>"

CALLBACK_POLICY_STATUS_BASE="/callbacks/policy-status"
CALLBACK_POLICY_STATUS=CALLBACK_POLICY_STATUS_BASE+"/<string:pid>"

STATUS="/status"

RESET="/reset"

# Supporteed job schema name
JOB_SCHEMA_NAME="STD_Weather_0.0.1"

#Consumer data
db={}

# Get status info
# URI and parameters (GET): "/status"
# -
@app.route(STATUS,
    methods=['GET'])
def status():
    global db
    return json.dumps(db),200


# Reset db
@app.route(RESET,
    methods=['POST'])
def reset():
    global db
    db={}
    return "",200


# Callback for job result
@app.route(CALLBACK_JOB_RESULT,
     methods=['POST'])
def callback_job_result(job_id):

    req_json = json.loads(request.data)
    print("Callback for job data: "+job_id + " data: "+json.dumps(req_json))
    create_polices(req_json)

    return "OK", 200

# Callback for job status
@app.route(CALLBACK_JOB_STATUS,
     methods=['POST'])
def callback_job_status(job_id):

    print("Callback for job status: "+job_id)

    return "OK", 200


# Callback for service inpms
@app.route(CALLBACK_SERVICE,
     methods=['POST'])
def callback_service(service_id):

    print("Callback for service " + service_id)

    if service_id != "rapp-test":
        return "Service id unknown", 400

    return "OK", 200


# Callback for policy status
@app.route(CALLBACK_POLICY_STATUS,
     methods=['POST'])
def callback_policy_status(pid):

    print("Status callback for policy: "+pid)

    return "OK", 200

# Create policies in all rics based on the received data
def create_polices(job_data):

    try:
        for ric in db["rics"].keys():
            ric_inst=db["rics"][ric]
            ric_name=ric_inst["name"]
            print("# "+json.dumps(ric_inst))
            types=ric_inst["types"]
            for ptype in types:
                print("## "+ptype)
                if ptype not in ric_inst["instances"].keys():
                    tmp_inst={}
                    ric_inst["instances"][ptype]=tmp_inst
                tmp_inst=ric_inst["instances"][ptype]
                for i in range(0,20):
                    pjson=None
                    pid="ee959f736ecc48309e5f4602271da_"+ric_name+"_"+ptype+"_"+str(i)
                    if ptype=="STD_QOS2_0.1.0":
                        level=30
                        if "t" in job_data.keys():
                            tlevel=int(job_data["t"])
                            level=level+tlevel
                        level=level+random.randint(0, 9)
                        pstring="{\"ric_id\":\""+ric_name+"\",\"policy_id\":\"" + pid +"\",\"service_id\":\"rapp-test\", \"policytype_id\":\""+ptype+"\", \"status_notification_uri\":\""+own_host+"/callbacks/policy/"+pid+"\", \"policy_data\":{ \"scope\": { \"ueId\": \"ue24"+str(i)+"\", \"qosId\": \"4\" }, \"qosObjectives\": { \"priorityLevel\":"+ str(level) + "} }}"
                        print(pstring)
                        pjson=json.loads(pstring)
                        tmp_inst[pid]=pjson
                    if ptype=="":
                        level=30
                        if "t" in job_data.keys():
                            tlevel=int(job_data["t"])
                            level=level+tlevel
                        level=level+random.randint(0, 9)
                        pstring="{\"ric_id\":\""+ric_name+"\",\"policy_id\":\"" + pid +"\",\"service_id\":\"rapp-test\", \"policytype_id\":\""+"\", \"status_notification_uri\":\""+own_host+"/callbacks/policy/"+pid+"\", \"policy_data\":{ \"scope\": { \"ueId\": \"ue24"+str(i)+"\", \"qosId\": \"4\" }, \"qosObjectives\": { \"priorityLevel\":"+ str(level) + "} }}"
                        print(pstring)
                        pjson=json.loads(pstring)
                        tmp_inst[pid]=pjson
                    if ptype=="2":
                        level=30
                        if "t" in job_data.keys():
                            tlevel=int(job_data["t"])
                            level=level+tlevel
                        level=level+random.randint(0, 9)
                        pstring="{\"ric_id\":\""+ric_name+"\",\"policy_id\":\"" + pid +"\",\"service_id\":\"rapp-test\", \"policytype_id\":\""+ptype+"\", \"status_notification_uri\":\""+own_host+"/callbacks/policy/"+pid+"\", \"policy_data\":{ \"scope\": { \"ueId\": \"ue24"+str(i)+"\", \"qosId\": \"4\" }, \"qosObjectives\": { \"priorityLevel\":"+ str(level) + "} }}"
                        print(pstring)
                        pjson=json.loads(pstring)
                        tmp_inst[pid]=pjson

    except Exception as err:
        if debug:
            traceback.print_exc()
        print("Error during create policies: "+ str(err))

# Create the job for data delivery
def create_job():

    print("Deploy policies")
    headers = {'content-type': 'application/json'}

    delete_job=True

    job_name=consumer_name+"-job-123"
    while True:
        try:
            #Delete old job, may contain incorrect data
            if delete_job:
                url=ecs_base+"/data-consumer/v1/info-jobs/"+job_name
                resp=requests.delete(url, verify=False, timeout=2)
                if resp.status_code == 200 or resp.status_code == 204:
                    print("job: "+ job_name+" deleted")

                delete_job=False

            print("Creating job")
            url=ecs_base+"/data-consumer/v1/info-types/"+JOB_SCHEMA_NAME
            print("Check job type "+JOB_SCHEMA_NAME+" on url "+url)
            resp=requests.get(url, verify=False, timeout=2)
            print("resp: "+str(resp.status_code))
            job_type=JOB_SCHEMA_NAME
            if resp.status_code == 200:
                print("Job schema "+JOB_SCHEMA_NAME + " exist")
            else:
                print("Job schema "+JOB_SCHEMA_NAME + " missing")
                job_type=None

            if job_type is not None:
                url=ecs_base+"/data-consumer/v1/info-jobs/"+job_name
                resp=requests.get(url, verify=False, timeout=2)
                if resp.status_code == 200:
                    print("Job " + job_name + " exist")
                else:
                    print("Creating job "+ job_name)
                    jstring="{\"jobOwner\": \""+consumer_name+"\",\"infoTypeId\": \""+JOB_SCHEMA_NAME+"\",\"jobResultUri\": \"" +own_host+CALLBACK_JOB_RESULT_BASE+"/"+job_name +"\",\"jobStatusNotificationUri\": \"" +own_host+CALLBACK_JOB_RESULT_BASE+"/"+job_name +"\",\"jobDefinition\": {\"long\":16.0,\"lat\":57.2,\"parameteter_names\": [\"t\",\"gust\"]}}"
                    jjson=json.loads(jstring)
                    print("Job create url: "+ url)
                    print("Job json: " + json.dumps(jjson))
                    resp=requests.put(url, json=jjson, headers=headers, verify=False, timeout=2)
                    print("resp: "+ resp.text)
                    if resp.status_code == 200:
                        print("Job " + job_name + " updated")
                    elif resp.status_code == 201:
                        print("Job " + job_name + "  created")
                    else:
                        print("Job " + job_name + " create/update failed:"+ str(resp.status_code))
        except Exception as err:
            if debug:
                traceback.print_exc()
            print("Error during Create job: "+ str(err))

        time.sleep(20)


def deploy_policies():

    print("Deploy policies")
    headers = {'content-type': 'application/json'}

    while True:
        try:
            pstring="{\"callback_url\": \""+own_host+"/callbacks/service/rapp-test"+"\",\"keep_alive_interval_seconds\": \"0\",\"service_id\": \"rapp-test\"}"
            pjson=json.loads(pstring)
            url=pms_base+"/a1-policy/v2/services"
            print("Registring server: "+pstring + " to url")
            resp=requests.put(url, json=pjson, headers=headers, verify=False, timeout=2)
            if resp.status_code == 200:
                print("Service rapp-test updated")
            elif resp.status_code == 201:
                print("Service rapp-test created")
            else:
                print("Service rapp-test put failed:"+ str(resp.status_code))

            print("## "+json.dumps(db))
            for ric in db["rics"].keys():
                ric_inst=db["rics"][ric]
                ric_name=ric_inst["name"]
                print("# "+json.dumps(ric_inst))
                types=ric_inst["types"]
                for ptype in types:
                    print("# "+ptype)
                    if ptype in ric_inst["instances"].keys():
                        tmp_inst=ric_inst["instances"][ptype]
                        for pid in tmp_inst.keys():
                            pjson=tmp_inst[pid]
                            url=pms_base+"/a1-policy/v2/policies?type="+ptype+"&id="+pid+"&ric="+ric_name+"&service=rapp-test"
                            if ptype=="":
                                url=pms_base+"/a1-policy/v2/policies?id="+pid+"&ric="+ric_name+"&service=rapp-test"
                            print("Policy create ulr: "+url)
                            resp=requests.put(url, json=pjson, headers=headers, verify=False, timeout=2)
                            if resp.status_code == 200:
                                print("Policy " + pid + " updated")
                            elif resp.status_code == 201:
                                print("Policy " + pid + " created")
                            else:
                                print("Policy " + pid + "create/update failed:"+ str(resp.status_code))
                                print(resp.text)
                        ric_inst["instances"][ptype]={}




        except Exception as err:
            if debug:
                traceback.print_exc()
            print("Error during create policies: "+ str(err))
        time.sleep(15)



# Network discovery of rics a and policy types
def network_discovery():

    print("Network discovery")

    headers = {'content-type': 'application/json'}

    while True:
        try:
            if "rics" not in db.keys():
                db["rics"]={}

            url=pms_base+"/rics"
            print("Network discovery on url: " +url)
            resp=requests.get(url, headers=headers, verify=False, timeout=2)
            if resp.status_code == 200:
                req_json = json.loads(resp.text)
                print("Ric data "+json.dumps(req_json))
                for ric in req_json:
                    ric_name=ric["ricName"]
                    if ric_name in db["rics"].keys():
                        db["rics"][ric_name]["types"]=ric["policyTypes"]
                        print("Updating "+ric_name+" with policytypes "+json.dumps(ric["policyTypes"]))
                    else:
                        tmp={}
                        db["rics"][ric_name]=tmp
                        db["rics"][ric_name]["instances"]={}
                        tmp["name"]=ric_name
                        tmp["types"]=ric["policyTypes"]
                        print("Adding "+ric_name+" with policytypes "+json.dumps(ric["policyTypes"]))
        except Exception as err:
            if debug:
                traceback.print_exc()
            print("Error during Network discovery: "+ str(err))
        time.sleep(10)



### Main function ###

# Set host port - only for local python execution from the cmd line

if len(sys.argv) > 1 :
    HOST_PORT=int(sys.argv[1])


pms_base=os.getenv('PMS_BASE')
if pms_base is None:
    sys.exit("Enviroment var PMS_BASE not set")
else:
    print("Enviroment var PMS_BASE = "+pms_base)

ecs_base=os.getenv('ECS_BASE')
if ecs_base is None:
    sys.exit("Enviroment var ECS_BASE not set")
else:
    print("Enviroment var ECS_BASE = "+ecs_base)

own_host=os.getenv('OWN_HOST')
if own_host is None:
    sys.exit("Enviroment var OWN_HOST not set")
else:
    print("Enviroment var OWN_HOST = "+own_host)

consumer_name=os.getenv('CONSUMER_NAME')
if consumer_name is None:
    sys.exit("Enviroment var CONSUMER_NAME not set")
else:
    print("Enviroment var CONSUMER_NAME = "+consumer_name)


thread_reg = threading.Thread(target=network_discovery, args=())
thread_reg.daemon = True
thread_reg.start()

thread_depl = threading.Thread(target=deploy_policies, args=())
thread_depl.daemon = True
thread_depl.start()

thread_data = threading.Thread(target=create_job, args=())
thread_data.daemon = True
thread_data.start()

if __name__ == "__main__":
    print("app.run on port: "+ str(HOST_PORT))
    app.run(port=HOST_PORT, host=HOST_IP)

