/*-
 * ========================LICENSE_START=================================
 * Copyright (C) 2021 Nordix Foundation. All rights reserved.
 * ======================================================================
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ========================LICENSE_END===================================
 */

package org.oransc.rappmanager.configuration;

import io.kubernetes.client.openapi.ApiClient;
import io.kubernetes.client.openapi.Configuration;
import io.kubernetes.client.openapi.apis.CoreV1Api;
import io.kubernetes.client.util.ClientBuilder;

import java.io.IOException;

import org.oransc.rappmanager.exception.ServiceException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

@Component
public class KubernetesClusterConfig {

    @Autowired
    private ApplicationConfig appConfig;

    private CoreV1Api api;

    private static final Logger logger = LoggerFactory.getLogger(KubernetesClusterConfig.class);

    private void connectToKubernetesCluster() throws ServiceException {
        logger.debug("Reading Kubernetes Configuration");
        ApiClient client;
        try {
            client = ClientBuilder.cluster().build();
            client.setBasePath(appConfig.getBasePath());
            client.setApiKey(appConfig.getToken());
            client.setSslCaCert(appConfig.getCert());
            Configuration.setDefaultApiClient(client);
            api = new CoreV1Api();
        } catch (IOException e) {
            throw new ServiceException("Failed to Read the Kubernetes Configuration", e);
        }
    }

    public CoreV1Api getKubernetesClient() throws ServiceException {
        if (api == null) {
            connectToKubernetesCluster();
        }
        return api;
    }
}
